@@ -23,9 +23,8 @@ Display an About window
 __author__ = 'Damon Lynch'
 __copyright__ = "Copyright 2016-2020, Damon Lynch"
 
-from gettext import gettext as _
 
-from PyQt5.QtCore import Qt, pyqtSlot
+from PyQt5.QtCore import Qt, pyqtSlot, QSize
 from PyQt5.QtGui import QPixmap, QFont
 
 from PyQt5.QtWidgets import (
@@ -35,7 +34,7 @@ from PyQt5.QtWidgets import (
 
 import raphodo.qrc_resources
 import raphodo.__about__ as __about__
-from raphodo.viewutils import translateButtons
+from raphodo.viewutils import translateDialogBoxButtons
 
 
 class AboutDialog(QDialog):
@@ -51,7 +50,17 @@ class AboutDialog(QDialog):
         self.setObjectName('AboutDialog')
         self.setStyleSheet('QDialog#AboutDialog {background-image: url(:/splashscreen.png);}')
         pixmap = QPixmap(':/splashscreen.png')
-        self.setFixedSize(pixmap.size())
+        try:
+            ratio = pixmap.devicePixelRatioF()
+        except AttributeError:
+            ratio = pixmap.devicePixelRatio()
+
+        if ratio > 1.0:
+            size = QSize(pixmap.width() / ratio, pixmap.height() / ratio)
+        else:
+            size = pixmap.size()
+
+        self.setFixedSize(size)
 
         # These values are derived from the splash screen image contents.
         # If the image changes, so should these
@@ -113,6 +122,7 @@ class AboutDialog(QDialog):
         Portions copyright © 2013 Bernard Baeyens.
         Portions copyright © 2012-2015 Jim Easterbrook.
         Portions copyright © 2012 Sven Marnach.
+        Portions copyright © 2015 Dmitry Shachnev.
 
         Photo %(photolink)s copyright © 2014-2018 Damon Lynch, all rights reserved.
         Camera icon courtesy %(artlink1)s.
@@ -121,6 +131,7 @@ class AboutDialog(QDialog):
         Speech bubble courtesy %(artlink4)s.
         Lightbulb icon courtesy %(artlink5)s.
         Unlink icon courtesy %(artlink6)s.
+        Clock icon courtesy %(artlink7)s.
 
         Translators:
 
@@ -189,7 +200,9 @@ class AboutDialog(QDialog):
             artlink4='<a href="http://www.iconsolid.com/" style="color: white;">Icons Solid</a>',
             artlink5='<a href="https://sellfy.com/designcoon" style="color: white;">Icon Coon</a>',
             artlink6='<a href="https://www.iconfinder.com/icons/1608708/unlink_icon" style="color: '
-                     'white;">Dave Gandy</a>'
+                     'white;">Dave Gandy</a>',
+            artlink7='<a href="https://www.flaticon.com/authors/pixel-perfect" style="color: '
+                     'white;">Pixel perfect</a>'
         )
 
         style_sheet = """QLabel {
@@ -224,7 +237,7 @@ class AboutDialog(QDialog):
 
         buttonBox = QDialogButtonBox()
         closeButton = buttonBox.addButton(QDialogButtonBox.Close)  # type: QPushButton
-        translateButtons(buttonBox)
+        translateDialogBoxButtons(buttonBox)
         self.creditsButton = buttonBox.addButton(
             _('Credits'), QDialogButtonBox.HelpRole
         )  # type: QPushButton
