@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2018 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2007-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -18,7 +18,7 @@
 # see <http://www.gnu.org/licenses/>.
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2007-2018, Damon Lynch"
+__copyright__ = "Copyright 2007-2020, Damon Lynch"
 
 from enum import (Enum, IntEnum)
 from PyQt5.QtCore import Qt
@@ -213,10 +213,11 @@ class FileType(IntEnum):
 class FileExtension(Enum):
     raw = 1
     jpeg = 2
-    other_photo = 3
-    video = 4
-    audio = 5
-    unknown = 6
+    heif = 3
+    other_photo = 4
+    video = 5
+    audio = 6
+    unknown = 7
 
 
 class FileSortPriority(IntEnum):
@@ -313,6 +314,8 @@ class ExtractionTask(Enum):
     extract_from_file = 9
     extract_from_file_and_load_metadata = 10
     load_from_exif_buffer = 11
+    load_heif_directly = 12
+    load_heif_and_exif_directly = 13
 
 
 class ExtractionProcessing(Enum):
@@ -376,7 +379,8 @@ ExtensionColorDict = {
     FileExtension.raw: CustomColors.color1,
     FileExtension.video: CustomColors.color2,
     FileExtension.jpeg: CustomColors.color4,
-    FileExtension.other_photo: CustomColors.color5,
+    FileExtension.heif: CustomColors.color5,
+    FileExtension.other_photo: CustomColors.color5
 }
 
 
@@ -497,7 +501,9 @@ class Desktop(Enum):
     ubuntugnome = 9
     popgnome = 10
     deepin = 11
-    unknown = 12
+    zorin = 12
+    ukui = 13
+    unknown = 14
 
 
 class FileManagerType(Enum):
@@ -505,15 +511,17 @@ class FileManagerType(Enum):
     select = 2
     dir_only_uri = 3
     show_item = 4
+    show_items = 5
 
 
 FileManagerBehavior = dict(
-    nautilus=FileManagerType.regular,
+    nautilus=FileManagerType.select,
     dolphin=FileManagerType.select,
     caja=FileManagerType.dir_only_uri,
     thunar=FileManagerType.dir_only_uri,
     nemo=FileManagerType.regular,
     pcmanfm=FileManagerType.dir_only_uri,
+    peony=FileManagerType.show_items
 )
 FileManagerBehavior['pcmanfm-qt'] = FileManagerType.dir_only_uri
 FileManagerBehavior['dde-file-manager'] = FileManagerType.show_item
@@ -531,26 +539,30 @@ DefaultFileBrowserFallback = dict(
     lxde='pcmanfm',
     lxqt='pcmanfm-qt',
     deepin='dde-file-manager',
+    kylin='peony',
 )
 
 
+# Sync with value in install.py
 class Distro(Enum):
     debian = 1
     ubuntu = 2
     fedora = 3
     neon = 4
     linuxmint = 5
-    korora = 6
+    zorin = 6
     arch = 7
     opensuse = 8
     manjaro = 9
     galliumos = 10
     peppermint = 11
-    antergos = 12
     elementary = 13
     centos = 14
-    gentoo = 15
-    deepin = 16
+    centos7 = 15
+    gentoo = 16
+    deepin = 17
+    kylin = 18
+    popos = 19
     unknown = 20
 
 
@@ -772,6 +784,20 @@ class Plural(Enum):
     two_form_plural = 3
 
 
+class ScalingAction(Enum):
+    turned_on = 1
+    not_set = 2
+    already_set = 3
+
+
+class ScalingDetected(Enum):
+    Qt = 1
+    Xsetting = 2
+    Qt_and_Xsetting = 3
+    undetected = 4
+
+
 # Use the character . to for download_name and path to indicate the user manually marked a
 # file as previously downloaded
-manually_marked_previously_downloaded = '.'
\ No newline at end of file
+manually_marked_previously_downloaded = '.'
+
