@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (C) 2016-2018 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2016-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -22,7 +22,7 @@ Dialog for editing download subfolder structure and file renaming
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2016-2018, Damon Lynch"
+__copyright__ = "Copyright 2016-2020, Damon Lynch"
 
 from typing import Dict, Optional, List, Union, Tuple, Sequence
 import webbrowser
@@ -30,7 +30,7 @@ import datetime
 import copy
 import logging
 
-from gettext import gettext as _
+
 
 from PyQt5.QtWidgets import (
     QTextEdit, QApplication, QComboBox, QPushButton, QLabel, QDialog, QDialogButtonBox,
@@ -55,7 +55,9 @@ from raphodo.preferences import DownloadsTodayTracker, Preferences, match_pref_l
 import raphodo.exiftool as exiftool
 from raphodo.utilities import remove_last_char_from_list_str
 from raphodo.messagewidget import MessageWidget
-from raphodo.viewutils import translateButtons
+from raphodo.viewutils import (
+    translateDialogBoxButtons, standardMessageBox, translateMessageBoxButtons
+)
 import raphodo.qrc_resources
 
 
@@ -422,6 +424,11 @@ def make_subfolder_menu_entry(prefs: Tuple[str]) -> str:
 
     desc = prefs[0]
     elements = prefs[1:]
+    # Translators: this text appears in menus and combo boxes. It displays the
+    # description of an item, and its elements.
+    # Translators: %(variable)s represents Python code, not a plural of the term
+    # variable. You must keep the %(variable)s untranslated, or the program will
+    # crash.
     return _("%(description)s - %(elements)s") % dict(
         description=desc, elements=os.sep.join(elements)
     )
@@ -437,6 +444,11 @@ def make_rename_menu_entry(prefs: Tuple[str]) -> str:
 
     desc = prefs[0]
     elements = prefs[1]
+    # Translators: this text appears in menus and combo boxes. It displays the
+    # description of an item, and its elements.
+    # Translators: %(variable)s represents Python code, not a plural of the term
+    # variable. You must keep the %(variable)s untranslated, or the program will
+    # crash.
     return _("%(description)s - %(elements)s") % dict(description=desc, elements=elements)
 
 
@@ -691,7 +703,7 @@ class CreatePreset(QDialog):
         buttonBox.addButton(QDialogButtonBox.Cancel)  # type: QPushButton
         self.saveButton = buttonBox.addButton(QDialogButtonBox.Save)  # type: QPushButton
         self.saveButton.setEnabled(False)
-        translateButtons(buttonBox)
+        translateDialogBoxButtons(buttonBox)
         buttonBox.rejected.connect(self.reject)
         buttonBox.accepted.connect(self.accept)
 
@@ -867,12 +879,18 @@ class PrefDialog(QDialog):
             # Translators: please do not modify, change the order of or leave out html formatting
             # tags like <i> and <b>. These are used to format the text the users sees.
             # In this case, the </i> really is supposed to come before the <i>.
+            # Translators: %(variable)s represents Python code, not a plural of the term
+            # variable. You must keep the %(variable)s untranslated, or the program will
+            # crash.
             subfolder_msg = _(
                 "The character</i> %(separator)s <i>creates a new subfolder level."
             ) % dict(separator=os.sep)
             # Translators: please do not modify, change the order of or leave out html formatting
             # tags like <i> and <b>. These are used to format the text the users sees
             # In this case, the </i> really is supposed to come before the <i>.
+            # Translators: %(variable)s represents Python code, not a plural of the term
+            # variable. You must keep the %(variable)s untranslated, or the program will
+            # crash.
             subfolder_first_char_msg = _(
                 "There is no need start or end with the folder separator </i> %(separator)s<i>, "
                 "because it is added automatically."
@@ -900,18 +918,26 @@ class PrefDialog(QDialog):
         self.example = QLabel()
 
         # Combobox with built-in and user defined presets
-        self.preset = PresetComboBox(prefs=prefs, preset_names=self.preset_names,
-                                     preset_type=self.preset_type, edit_mode=True)
+        self.preset = PresetComboBox(
+            prefs=prefs, preset_names=self.preset_names, preset_type=self.preset_type,
+            edit_mode=True
+        )
         self.preset.activated.connect(self.presetComboItemActivated)
 
-        flayout = QFormLayout()
-        flayout.addRow(_('Preset:'), self.preset)
-        flayout.addRow(_('Example:'), self.example)
+        glayout = QGridLayout()
+        presetLabel = QLabel(_('Preset:'))
+        exampleLabel = QLabel(_('Example:'))
+
+        glayout.addWidget(presetLabel, 0, 0)
+        glayout.addWidget(self.preset, 0, 1)
+        glayout.addWidget(exampleLabel, 1, 0)
+        glayout.addWidget(self.example, 1, 1)
+        glayout.setColumnStretch(1, 10)
 
         layout = QVBoxLayout()
         self.setLayout(layout)
 
-        layout.addLayout(flayout)
+        layout.addLayout(glayout)
         layout.addSpacing(int(QFontMetrics(QFont()).height() / 2))
         layout.addWidget(self.editor)
         layout.addWidget(self.messageWidget)
@@ -1029,7 +1055,7 @@ class PrefDialog(QDialog):
         self.helpButton = buttonBox.button(QDialogButtonBox.Help)  # type: QPushButton
         self.helpButton.clicked.connect(self.helpButtonClicked)
         self.helpButton.setToolTip(_('Get help online...'))
-        translateButtons(buttonBox)
+        translateDialogBoxButtons(buttonBox)
 
         buttonBox.rejected.connect(self.reject)
         buttonBox.accepted.connect(self.accept)
@@ -1392,12 +1418,9 @@ class PrefDialog(QDialog):
         """
 
         if self.preset.preset_edited or self.preset.new_preset:
-            msgBox = QMessageBox()
             title = _("Save Preset - Rapid Photo Downloader")
-            msgBox.setTextFormat(Qt.RichText)
-            msgBox.setIcon(QMessageBox.Question)
-            msgBox.setWindowTitle(title)
             if self.preset.new_preset:
+
                 message = _(
                     "<b>Do you want to save the changes in a new custom preset?</b><br><br>"
                     "Creating a custom preset is not required, but can help you keep "
@@ -1405,10 +1428,17 @@ class PrefDialog(QDialog):
                     "The changes to the preferences will still be applied regardless of "
                     "whether you create a new custom preset or not."
                 )
-                msgBox.setStandardButtons(QMessageBox.Yes|QMessageBox.No)
+                msgBox = standardMessageBox(
+                    standardButtons=QMessageBox.Yes | QMessageBox.No,
+                    title=title, rich_text=True, message=message
+                )
                 updateButton = newButton = None
             else:
                 assert self.preset.preset_edited
+                msgBox = QMessageBox()
+                msgBox.setTextFormat(Qt.RichText)
+                msgBox.setIcon(QMessageBox.Question)
+                msgBox.setWindowTitle(title)
                 message = _(
                     "<b>Do you want to save the changes in a custom preset?</b><br><br>"
                     "If you like, you can create a new custom preset or update the "
@@ -1416,13 +1446,14 @@ class PrefDialog(QDialog):
                     "The changes to the preferences will still be applied regardless of "
                     "whether you save a custom preset or not."
                 )
+                msgBox.setText(message)
+                msgBox.addButton(QMessageBox.No)
+                translateMessageBoxButtons(msgBox)
                 updateButton = msgBox.addButton(
                     _('Update Custom Preset "%s"') % self.current_custom_name, QMessageBox.YesRole
                 )
                 newButton = msgBox.addButton(_('Save New Custom Preset'), QMessageBox.YesRole)
-                msgBox.addButton(QMessageBox.No)
 
-            msgBox.setText(message)
             choice = msgBox.exec()
             save_new = update = False
             if self.preset.new_preset:
