@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2019 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2017-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -22,7 +22,7 @@ to download them.
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2017-2019, Damon Lynch"
+__copyright__ = "Copyright 2017-2020, Damon Lynch"
 
 import logging
 from collections import namedtuple
@@ -33,7 +33,6 @@ import traceback
 import shutil
 from typing import Optional
 
-from gettext import gettext as _
 import requests
 
 import arrow
@@ -47,9 +46,9 @@ from raphodo.constants import (
     remote_versions_file, CheckNewVersionDialogState, CheckNewVersionDialogResult,
     standardProgressBarWidth
 )
-from raphodo.utilities import (create_temp_dir, format_size_for_user, is_venv)
+from raphodo.utilities import (create_temp_dir, format_size_for_user, is_venv, installed_using_pip)
 from raphodo.interprocess import ThreadNames
-from raphodo.viewutils import translateButtons
+from raphodo.viewutils import translateDialogBoxButtons
 
 version_details = namedtuple('version_details', 'version release_date url md5 changelog_url')
 
@@ -88,25 +87,6 @@ class NewVersion(QObject):
         self.thread_controller = context.socket(zmq.PAIR)
         self.thread_controller.connect('inproc://{}'.format(ThreadNames.new_version))
 
-    def installedUsingPip(self, package='rapid-photo-downloader') -> bool:
-        """
-        Determine if python package was installed using pip.
-
-        Exceptions are not caught.
-
-        :param package: package name to search for
-        :return: True if installed via pip, else False
-        """
-
-        pip_install = False
-        pkg = pkg_resources.get_distribution(package)
-        if pkg.has_metadata('INSTALLER'):
-            if pkg.get_metadata('INSTALLER').strip() == 'pip':
-                pip_install = True
-
-        logging.debug("Installed using pip: %s", pip_install)
-        return pip_install
-
     @pyqtSlot()
     def check(self) -> None:
         success = False
@@ -154,7 +134,7 @@ class NewVersion(QObject):
 
         if not self.installed_via_pip_check_made:
             try:
-                self.installed_via_pip = self.installedUsingPip()
+                self.installed_via_pip = installed_using_pip()
             except Exception:
                 logging.debug(
                     "Exception encountered when checking if pip was used to install "
@@ -340,11 +320,11 @@ class NewVersionCheckDialog(QDialog):
         self.messages.addWidget(self.failedToCheck)
 
         cancelBox = QDialogButtonBox(QDialogButtonBox.Cancel)
-        translateButtons(cancelBox)
+        translateDialogBoxButtons(cancelBox)
         cancelBox.rejected.connect(self.reject)
 
         self.downloadItBox = QDialogButtonBox(QDialogButtonBox.Yes | QDialogButtonBox.No)
-        translateButtons(self.downloadItBox)
+        translateDialogBoxButtons(self.downloadItBox)
         # Translators: this text appears in a button - the & sets the s key in combination with
         # the alt key to act as the keyboard shortcut
         self.dlItSkipButton = QPushButton(_('&Skip this release'))
@@ -355,11 +335,11 @@ class NewVersionCheckDialog(QDialog):
         self.downloadItBox.clicked.connect(self.downloadItClicked)
 
         closeBox = QDialogButtonBox(QDialogButtonBox.Close)
-        translateButtons(closeBox)
+        translateDialogBoxButtons(closeBox)
         closeBox.rejected.connect(self.reject)
 
         openDownloadPageBox = QDialogButtonBox(QDialogButtonBox.Close)
-        translateButtons(openDownloadPageBox)
+        translateDialogBoxButtons(openDownloadPageBox)
         # Translators: this text appears in a button - the & sets the s key in combination with
         # the alt key to act as the keyboard shortcut
         self.openDlPageSkipButton = QPushButton(_('&Skip this release'))
@@ -487,6 +467,9 @@ class DownloadNewVersionDialog(QDialog):
         bytes_downloaded_display = format_size_for_user(bytes_downloaded, zero_string='0 KB')
 
         # Translators: shows how much of a file has been downloaded e.g  123 KB of 1.3 MB
+        # Translators: %(variable)s represents Python code, not a plural of the term
+        # variable. You must keep the %(variable)s untranslated, or the program will
+        # crash.
         self.text = _('%(downloaded)s of %(total)s')
         self.message = QLabel(
             self.text % dict(downloaded=bytes_downloaded_display, total=self.download_size_display)
@@ -498,7 +481,7 @@ class DownloadNewVersionDialog(QDialog):
         self.progressBar.setValue(bytes_downloaded)
 
         buttonBox = QDialogButtonBox(QDialogButtonBox.Cancel)
-        translateButtons(buttonBox)
+        translateDialogBoxButtons(buttonBox)
         buttonBox.rejected.connect(self.reject)
 
         grid = QGridLayout()
