@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2011-2019 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2011-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -27,7 +27,7 @@ which is what a gphoto2 process does.
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2019, Damon Lynch"
+__copyright__ = "Copyright 2011-2020, Damon Lynch"
 
 from typing import List, DefaultDict, Optional
 import logging
@@ -55,7 +55,11 @@ class RescanCamera:
 
     def __init__(self, camera: Camera, prefs: Preferences) -> None:
         self.camera = camera
-        assert camera.camera_has_dcim_like_folder()
+        if not camera.specific_folder_located:
+            logging.warning(
+                "No folders located on %s: there might be a bug the camera firmware or libgphoto2. "
+                "Continuing rescan regardless.", camera.display_name
+            )
         # Relocated RPD files
         self.rpd_files = []  # type: List[RPDFile]
         # Missing RPD files
