@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2018 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2011-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -17,7 +17,7 @@
 # see <http://www.gnu.org/licenses/>.
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2018, Damon Lynch"
+__copyright__ = "Copyright 2011-2020, Damon Lynch"
 
 import os
 import time
@@ -29,7 +29,6 @@ from collections import Counter, UserDict
 import locale
 from typing import Optional, List, Tuple, Union, Any
 
-from gettext import gettext as _
 import gi
 gi.require_version('GLib', '2.0')
 from gi.repository import GLib
@@ -249,6 +248,9 @@ class FileTypeCounter(Counter):
          present e.g. "photos and videos"
         """
         file_types_present = self.file_types_present()
+        # Translators: %(variable)s represents Python code, not a plural of the term
+        # variable. You must keep the %(variable)s untranslated, or the program will
+        # crash.
         file_count_summary = _("%(number)s %(filetypes)s") % dict(
             number=thousands(self[FileType.photo] + self[FileType.video]),
             filetypes=file_types_present
@@ -268,15 +270,21 @@ class FileTypeCounter(Counter):
         v = self[FileType.video]
 
         if v > 1:
-            videos =  _('%(no_videos)s Videos') % dict(no_videos=thousands(v))
+            # Translators: %(variable)s represents Python code, not a plural of the term
+            # variable. You must keep the %(variable)s untranslated, or the program will
+            # crash.
+            videos = _('%(no_videos)s Videos') % dict(no_videos=thousands(v))
         elif v == 1:
             if singular_natural:
                 # translators: natural language expression signifying a single video
                 videos = _('a video')
             else:
-                videos =  _('1 Video')
+                videos = _('1 Video')
 
         if p > 1:
+            # Translators: %(variable)s represents Python code, not a plural of the term
+            # variable. You must keep the %(variable)s untranslated, or the program will
+            # crash.
             photos = _('%(no_photos)s Photos') % dict(no_photos=thousands(p))
         elif p == 1:
             if singular_natural:
@@ -427,7 +435,7 @@ class RPDFile:
         # If None, haven't tried yet
         # If False, no problems encountered, got it (or it was assigned from mtime
         # when never_read_mdatatime is True)
-        self._no_datetime_metadata = None  #type: Optional[bool]
+        self._no_datetime_metadata = None  # type: Optional[bool]
 
         self.never_read_mdatatime = never_read_mdatatime
         if never_read_mdatatime:
@@ -673,7 +681,6 @@ class RPDFile:
         """
         return self.mime_type == 'image/jpeg'
 
-
     def is_jpeg_type(self) -> bool:
         """
         :return:True if the image is a jpeg or MPO image
@@ -694,6 +701,13 @@ class RPDFile:
         """
         return self.extension in fileformats.RAW_EXTENSIONS
 
+    def is_heif(self) -> bool:
+        """
+        Inspects file extension to determine if an HEIF / HEIC file
+        :return:
+        """
+        return self.extension in fileformats.HEIF_EXTENTIONS
+
     def is_tiff(self) -> bool:
         """
         :return: True if the file is a tiff file
@@ -728,7 +742,6 @@ class RPDFile:
         # take advantage of Python's left to right evaluation:
         return self.temp_sample_full_file_name or self.get_current_full_file_name()
 
-
     def get_current_name(self) -> str:
         """
         :return: file name which depending on download status will be the
@@ -782,6 +795,8 @@ class RPDFile:
         """
 
         if self.from_camera:
+            # Translators: %(variable)s represents Python code, not a plural of the term variable.
+            # You must keep the %(variable)s untranslated, or the program will crash.
             return _('%(path)s on %(camera)s') % dict(
                 path=self.full_file_name, camera=self.camera_display_name
             )
@@ -807,9 +822,10 @@ class Photo(RPDFile):
         self.file_type = FileType.photo
 
     def load_metadata(self, full_file_name: Optional[str]=None,
-                 raw_bytes: Optional[bytearray]=None,
-                 app1_segment: Optional[bytearray]=None,
-                 et_process: exiftool.ExifTool=None) -> bool:
+                      raw_bytes: Optional[bytearray]=None,
+                      app1_segment: Optional[bytearray]=None,
+                      et_process: exiftool.ExifTool=None,
+                      force_exiftool: Optional[bool] = False) -> bool:
         """
         Use GExiv2 or ExifTool to read the photograph's metadata.
 
@@ -820,10 +836,14 @@ class Photo(RPDFile):
         :param app1_segment: the app1 segment of a jpeg file, from which
          the metadata can be read
         :param et_process: optional daemon ExifTool process
+        :param force_exiftool: whether ExifTool must be used to load the
+         metadata
         :return: True if successful, False otherwise
         """
 
-        if fileformats.use_exiftool_on_photo(self.extension):
+        if force_exiftool or fileformats.use_exiftool_on_photo(
+                self.extension, preview_extraction_irrelevant=True):
+
             self.metadata = metadataexiftool.MetadataExiftool(
                 full_file_name=full_file_name, et_process=et_process, file_type=self.file_type
             )
