@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2019 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2007-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -18,7 +18,7 @@
 # see <http://www.gnu.org/licenses/>.
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2007-2019, Damon Lynch"
+__copyright__ = "Copyright 2007-2020, Damon Lynch"
 
 import contextlib
 import locale
@@ -32,21 +32,27 @@ import tempfile
 import time
 import tarfile
 from collections import namedtuple, defaultdict
+
 from datetime import datetime
-from gettext import gettext as _
 from itertools import groupby, zip_longest
-from typing import Optional, List, Union, Any, Tuple
+from typing import Optional, List, Union, Any, Tuple, Iterator
 import struct
 import ctypes
 import signal
 import warnings
+import xdg
+import babel
+from glob import glob
 from pkg_resources import parse_version
+import pkg_resources
 
 import arrow
 import psutil
-from PyQt5.QtCore import QSize
+from PyQt5.QtCore import QSize, QLocale, QTranslator, QLibraryInfo
 
 import raphodo.__about__ as __about__
+from raphodo.constants import disable_version_check
+from raphodo import localedir, i18n_domain
 
 
 # Arrow 0.9.0 separated the replace and shift functions into separate calls, deprecating using
@@ -67,6 +73,7 @@ else:
     except AttributeError:
         arrow_shift_support = False
 
+
 # Suppress parsing warnings for 0.14.3 <= Arrow version < 0.15
 if arrow_version >= parse_version('0.14.3') and arrow_version < parse_version('0.15.0'):
     from arrow.factory import ArrowParseWarning
@@ -419,17 +426,26 @@ def make_internationalized_list(items: List[str]) -> str:
     if len(items) == 1:
         return items[0]
     if len(items) == 2:
-        # two things in a list e.g. "device1 and device2"
+        # Translators: two things in a list e.g. "device1 and device2"
+        # Translators: %(variable)s represents Python code, not a plural of the term
+        # variable. You must keep the %(variable)s untranslated, or the program will
+        # crash.
         return _('%(first_item)s and %(last_item)s') % dict(
             first_item=items[0], last_item=items[1])
     if len(items) > 2:
         s = items[0]
         for item in items[1:-1]:
-            # the middle of a list of things
+            # Translators: the middle of a list of things
+            # Translators: %(variable)s represents Python code, not a plural of the term
+            # variable. You must keep the %(variable)s untranslated, or the program will
+            # crash.
             s =  '%(first_items)s, %(last_items)s'% dict(
                 first_items=s, last_items=item
             )
-        # the end of a list of things
+        # Translators: the end of a list of things
+        # Translators: %(variable)s represents Python code, not a plural of the term
+        # variable. You must keep the %(variable)s untranslated, or the program will
+        # crash.
         s = '%(start_items)s and %(last_item)s' % dict(
             start_items=s, last_item=items[-1]
         )
@@ -446,7 +462,7 @@ def thousands(i: int) -> str:
     :return: string with seperators e.g. '1,000'
     """
     try:
-        return locale.format("%d", i, grouping=True)
+        return locale.format_string("%d", i, grouping=True)
     except TypeError:
         return i
 
@@ -493,8 +509,10 @@ def runs(iterable):
     for k, g in groupby(iterable, AdjacentKey):
         yield first_and_last(g)
 
+
 numbers = namedtuple('numbers', 'number, plural')
 
+
 long_numbers = {
     1: _('one'),
     2: _('two'),
@@ -570,6 +588,7 @@ def datetime_roughly_equal(dt1: Union[datetime, float], dt2: Union[datetime, flo
     True
     """
 
+    # arrow.get from time stamp gives UTC time
     at1 = arrow.get(dt1)
     at2 = arrow.get(dt2)
     if arrow_shift_support:
@@ -603,6 +622,7 @@ def process_running(process_name: str, partial_name: bool=True) -> bool:
                     return True
     return False
 
+
 def make_html_path_non_breaking(path: str) -> str:
     """
     When /some/path is displayed in rich text, it will be word-wrapped on the
@@ -838,6 +858,78 @@ def extract_file_from_tar(full_tar_path, member_filename) -> bool:
             return False
 
 
+def bug_report_full_tar_path() -> str:
+    """
+    Generate a full path for uncompressed bug report tar file.
+    The filename will not already exist.
+
+    :return: File name including path
+    """
+
+    filename = 'rpd-bug-report-{}'.format(datetime.now().strftime('%Y%m%d'))
+    component = os.path.join(os.path.expanduser('~'), filename)
+
+    i = 0
+    while os.path.isfile('{}{}.tar'.format(component, '' if not i else '-{}'.format(i))):
+        i += 1
+
+    return '{}{}.tar'.format(component, '' if not i else '-{}'.format(i))
+
+
+def create_bugreport_tar(full_tar_name: str,
+                         log_path: Optional[str]='',
+                         full_config_file: Optional[str]='') -> bool:
+    """
+    Create a tar file containing log and configuration files.
+
+    If the file already exists, do nothing.
+
+    :param full_tar_name: the full path in which to create the tar file
+    :param log_path: path to the log files
+    :param full_config_file: the full path and file of the configuration file
+    :return: True if tar file created, else False
+    """
+
+    if os.path.isfile(full_tar_name):
+        logging.error("Cannot create bug report tarfile, because it already exists")
+        return False
+
+    if not log_path:
+        log_path = os.path.join(xdg.BaseDirectory.xdg_cache_home, 'rapid-photo-downloader', 'log')
+
+    if not full_config_file:
+        config_dir = os.path.join(xdg.BaseDirectory.xdg_config_home, 'Rapid Photo Downloader')
+        config_file = 'Rapid Photo Downloader.conf'
+    else:
+        config_dir, config_file = os.path.split(full_config_file)
+
+    curr_dir = os.getcwd()
+    created = False
+
+    try:
+        with tarfile.open(full_tar_name, 'x') as t:
+            os.chdir(log_path)
+            for l in glob('*'):
+                t.add(l)
+            os.chdir(config_dir)
+            t.add(config_file)
+    except FileNotFoundError as e:
+        logging.error(
+            "When creating a bug report tar file, the directory or file %s does not exist", e.filename
+        )
+    except Exception:
+        logging.exception("Unexpected error when creating bug report tar file")
+    else:
+        created = True
+
+    try:
+        os.chdir(curr_dir)
+    except FileNotFoundError:
+        pass
+
+    return created
+
+
 def current_version_is_dev_version(current_version=None) -> bool:
     if current_version is None:
         current_version = parse_version(__about__.__version__)
@@ -850,17 +942,18 @@ def remove_topmost_directory_from_path(path: str) -> str:
     return path[path[1:].find(os.sep) + 1:]
 
 
-def arrow_locale() -> str:
+def arrow_locale(lang: str) -> str:
     """
     Test if locale is suitable for use with Arrow.
     :return: Return user locale if it works with Arrow, else Arrow default ('en_us')
     """
 
     default = 'en_us'
-    try:
-        lang = locale.getdefaultlocale()[0]
-    except Exception:
-        return default
+    if not lang:
+        try:
+            lang = locale.getdefaultlocale()[0]
+        except Exception:
+            return default
 
     try:
         arrow.locales.get_locale(lang)
@@ -980,4 +1073,202 @@ def is_venv():
     """
 
     return hasattr(sys, 'real_prefix') or \
-           (hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix)
\ No newline at end of file
+           (hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix)
+
+
+def python_package_version(package: str) -> str:
+    """
+    Determine the version of an installed Python package
+    :param package: package name
+    :return: version number, if could be determined, else ''
+    """
+
+    try:
+        return pkg_resources.get_distribution(package).version
+    except pkg_resources.DistributionNotFound:
+        return ''
+
+
+def is_snap() -> bool:
+    """
+    Determine if program is running in a snap environment
+    :return: True if it is False otherwise
+    """
+
+    snap_name = os.getenv('SNAP_NAME', '')
+    return snap_name.find('rapid-photo-downloader') >= 0
+
+
+def version_check_disabled():
+    """
+    Determine if version checking should be disabled or not
+    :return: True if it should be False otherwise
+    """
+
+    return disable_version_check or is_snap()
+
+
+def available_lang_codes() -> List[str]:
+    """
+    Detect translations that exist for Rapid Photo Downloader
+    :return: list of language codes
+    """
+
+    if localedir is not None:
+        files = glob(os.path.join(localedir, '*', 'LC_MESSAGES', '%s.mo' % i18n_domain))
+        langs = [file.split(os.path.sep)[-3] for file in files]
+        langs.append('en')
+        return langs
+    else:
+        return []
+
+
+# Auto-generated from extract_language_names.py do not delete
+substitute_languages = {
+    'fa': 'Persian',
+    'sk': 'Slovak',
+    'it': 'Italian',
+    'oc': 'Occitan (post 1500)',
+    'fi': 'Finnish',
+    'sv': 'Swedish',
+    'cs': 'Czech',
+    'pl': 'Polish',
+    'kab': 'Kabyle',
+    'tr': 'Turkish',
+    'hr': 'Croatian',
+    'nn': 'Norwegian Nynorsk',
+    'da': 'Danish',
+    'de': 'German',
+    'sr': 'српски',
+    'pt_BR': 'Brazilian Portuguese',
+    'ja': 'Japanese',
+    'bg': 'Bulgarian',
+    'uk': 'Ukrainian',
+    'ar': 'Arabic',
+    'ca': 'Catalan',
+    'nb': 'Norwegian Bokmal',
+    'ru': 'Russian',
+    'hu': 'magyar',
+    'be': 'Belarusian',
+    'es': 'Spanish',
+    'pt': 'Portuguese',
+    'zh_CN': 'Chinese (Simplified)',
+    'fr': 'Français',
+    'et': 'Estonian',
+    'nl': 'Dutch',
+    'ro': 'Romanian',
+    'id': 'Indonesian',
+    'el': 'Greek',
+}  # Auto-generated from extract_language_names.py do not delete
+
+
+def get_language_display_name(lang_code: str,
+                              make_missing_lower: bool,
+                              locale_code: str) -> str:
+    """
+    Use babel to the human friendly name for a locale, or failing that our
+    auto-generated version
+    :param lang_code: locale code for language to get the display name for
+    :param make_missing_lower: whether to make the default name when
+     babel does not suppply it lower case
+    :param locale_code: current system locale code
+    :return: human friendly version
+    """
+
+    try:
+        return babel.Locale.parse(lang_code).get_display_name(locale_code)
+    except babel.core.UnknownLocaleError:
+        display = substitute_languages[lang_code]
+        return display if not make_missing_lower else display.lower()
+
+
+def available_languages(display_locale_code: str='') -> List[Tuple[str, str]]:
+    """
+    Detect translations that exist for Rapid Photo Downloader
+    :return: iterator of Tuple of language code and localized name
+    """
+
+    lang_codes = available_lang_codes()
+
+    if not lang_codes:  # Testing code when translations are not installed
+        lang_codes = ['en', 'de', 'es']
+
+    if not display_locale_code:
+        try:
+            locale_code = locale.getdefaultlocale()[0]
+        except Exception:
+            locale_code = 'en_US'
+    else:
+        locale_code = display_locale_code
+
+    # Determine if this locale makes its language names lower case
+    babel_sample = babel.Locale.parse('en').get_display_name(locale_code)
+    make_missing_lower = babel_sample.islower()
+
+    langs = zip(
+        lang_codes, [
+            get_language_display_name(code, make_missing_lower, locale_code) for code in lang_codes
+        ]
+    )
+
+    # Sort languages by display name
+    langs = list(langs)
+    try:
+        langs.sort(key=lambda l: locale.strxfrm(l[1]))
+    except Exception:
+        logging.error("Error sorting language names for display in program preferences")
+    return langs
+
+
+def installed_using_pip(package='rapid-photo-downloader') -> bool:
+    """
+    Determine if python package was installed using pip.
+
+    Exceptions are not caught.
+
+    :param package: package name to search for
+    :return: True if installed via pip, else False
+    """
+
+    pip_install = False
+    pkg = pkg_resources.get_distribution(package)
+    if pkg.has_metadata('INSTALLER'):
+        if pkg.get_metadata('INSTALLER').strip() == 'pip':
+            pip_install = True
+
+    return pip_install
+
+
+def getQtSystemTranslation(locale_name: str) -> Optional[QTranslator]:
+    """
+    Attempt to install Qt base system translations (for QMessageBox and QDialogBox buttons)
+    :return: translator if loaded, else None
+    """
+
+    # These locales are found in the path QLibraryInfo.TranslationsPath
+    convert_locale = dict(
+        cs_CZ='cs',
+        da_DK='da',
+        de_DE='de',
+        es_ES='es',
+        fi_FI='fi',
+        fr_FR='fr',
+        it_IT='it',
+        ja_JP='ja',
+        hu_HU='hu',
+        pl_PL='pl',
+        ru_RU='ru',
+        sk_SK='sk',
+        uk_UA='uk',
+    )
+
+    qtTranslator = QTranslator()
+    location = QLibraryInfo.location(QLibraryInfo.TranslationsPath)
+    qm_file = "qtbase_{}.qm".format(convert_locale.get(locale_name, locale_name))
+    qm_file = os.path.join(location, qm_file)
+    if os.path.isfile(qm_file):
+        if qtTranslator.load(qm_file):
+            logging.debug("Installing Qt support for locale %s", locale_name)
+            return qtTranslator
+        else:
+            logging.debug("Could not load Qt locale file %s", qm_file)
